@@ -49,15 +49,11 @@ using namespace std;
 #define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 //#define debug(x)
 
-CreateScenario::CreateScenario()
+CreateScenario::CreateScenario(int width, int height)
     :d_scenario(0), d_generator(0)
 {
     debug("CreateScenario::CreateScenario")
    
-   //default value
-    setWidth(112);
-    setHeight(156);
-    
     //make sure that objects are deleted
     GameMap::deleteInstance();
     Playerlist::deleteInstance();
@@ -96,6 +92,8 @@ CreateScenario::CreateScenario()
         std::cerr <<"CreateScenario: Didn't succeed in reading object names. Aborting!\n";
         exit(-1);
     }
+    setWidth(width);
+    setHeight(height);
 }
 
 CreateScenario::~CreateScenario()
@@ -232,7 +230,8 @@ Player* CreateScenario::addPlayer(std::string name, Uint32 armyset,
 {
     debug("CreateScenario::addPlayer")
 
-    Player* p = Player::create(name, armyset, color, Player::Type(type));
+    Player* p = Player::create(name, armyset, color, d_width, d_height,
+			       Player::Type(type));
     Playerlist::getInstance()->push_back(p);
 
     return p;
@@ -310,7 +309,9 @@ bool CreateScenario::create(const GameParameters &g)
     GameScenario::s_play_with_quests = g.play_with_quests;
     GameScenario::s_hidden_map = g.hidden_map;
     GameScenario::s_diplomacy = g.diplomacy;
+    GameScenario::s_cusp_of_war = g.cusp_of_war;
     GameScenario::s_neutral_cities = g.neutral_cities;
+    GameScenario::s_razing_cities = g.razing_cities;
     GameScenario::s_military_advisor= g.military_advisor;
     GameScenario::s_random_turns = g.random_turns;
     GameScenario::s_intense_combat = g.intense_combat;
@@ -341,7 +342,7 @@ bool CreateScenario::create(const GameParameters &g)
     if (!setupSignposts())
         return false;
 
-    if (!setupPlayers())
+    if (!setupPlayers(g.diplomacy, g.random_turns))
         return false;
 
     if (!setupRewards())
@@ -438,9 +439,9 @@ bool CreateScenario::distributePlayers()
             (*cit).setCapital(true);
             skipping = 0;
 
-            History_CityWon *item = new History_CityWon();
-            item->fillData(&*cit);
-            (*pit)->getHistorylist()->push_back(item);
+	    History_CityWon *item = new History_CityWon();
+	    item->fillData(&*cit);
+	    (*pit)->getHistorylist()->push_back(item);
 
             pit++;
             if ((*pit) == pl->getNeutral())
@@ -460,39 +461,7 @@ bool CreateScenario::setupCities(bool quick_start,
     debug("CreateScenario::setupCities")
 
     if (quick_start)
-      {
-        Playerlist *plist = Playerlist::getInstance();
-        Citylist *clist = Citylist::getInstance();
-        Vector <int> pos;
-        // no neutral cities
-        // divvy up the neutral cities among other non-neutral players
-        int cities_left = clist->size() - plist->size() + 1;
-        unsigned int citycount[MAX_PLAYERS];
-	memset (citycount, 0, sizeof (citycount));
-        Playerlist::iterator pit = plist->begin();
-        for (; pit != plist->end(); pit++)
-          {
-            if (*pit == plist->getNeutral())
-              pit = plist->begin();
-            citycount[(*pit)->getId()]++;
-            cities_left--;
-            if (cities_left == 0)
-              break;
-          }
-        for (unsigned int i = 0; i < MAX_PLAYERS; i++)
-          {
-            for (unsigned int j = 0; j < citycount[i]; j++)
-              {
-                Player *p = plist->getPlayer(i);
-                pos = clist->getFirstCity(p)->getPos();
-	        City *c = clist->getNearestNeutralCity(pos);
-                c->conquer(p);
-                History_CityWon *item = new History_CityWon();
-                item->fillData(c);
-                p->getHistorylist()->push_back(item);
-              }
-          }
-      }
+      quickStart();
 
     for (Citylist::iterator it = Citylist::getInstance()->begin();
         it != Citylist::getInstance()->end(); it++)
@@ -704,16 +673,44 @@ bool CreateScenario::setupSignposts()
     return true;
 }
 
-bool CreateScenario::setupPlayers()
+bool CreateScenario::setupPlayers(bool diplomacy, bool random_turns)
 {
     debug("CreateScenario::setupPlayers")
 
-    Playerlist *plist = Playerlist::getInstance();
-    Playerlist::iterator pit = plist->begin();
-    for (; pit != plist->end(); pit++)
+    Playerlist *pl = Playerlist::getInstance();
+
+    // Give players some gold to start with
+    for (Playerlist::iterator pit = pl->begin(); pit != pl->end(); pit++)
+      (*pit)->setGold(1000 + ((rand() % 8) * 50));
+
+    // Set up diplomacy
+    for (Playerlist::iterator pit = pl->begin(); pit != pl->end(); pit++)
       {
-	(*pit)->setGold(1000 + ((rand() % 8) * 50));
+	if (pl->getNeutral() == (*pit))
+	  continue;
+	for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
+	  {
+	    if (pl->getNeutral() == (*it))
+	      continue;
+	    if (*pit == *it)
+	      continue;
+	    if (diplomacy == false)
+	      {
+		(*pit)->proposeDiplomacy(Player::PROPOSE_WAR, *it);
+		(*pit)->declareDiplomacy(Player::AT_WAR, *it);
+	      }
+	    else 
+	      {
+		(*pit)->proposeDiplomacy(Player::NO_PROPOSAL, *it);
+		(*pit)->declareDiplomacy(Player::AT_PEACE, *it);
+	      }
+	  }
       }
+    if (diplomacy)
+      pl->calculateDiplomaticRankings();
+
+    if (random_turns)
+      pl->randomizeOrder();
     return true;
 }
 
@@ -842,3 +839,37 @@ bool CreateScenario::setupItemRewards()
   return true;
 }
 
+void CreateScenario::quickStart()
+{
+  Playerlist *plist = Playerlist::getInstance();
+  Citylist *clist = Citylist::getInstance();
+  Vector <int> pos;
+  // no neutral cities
+  // divvy up the neutral cities among other non-neutral players
+  int cities_left = clist->size() - plist->size() + 1;
+  unsigned int citycount[MAX_PLAYERS];
+  memset (citycount, 0, sizeof (citycount));
+  Playerlist::iterator pit = plist->begin();
+  for (; pit != plist->end(); pit++)
+    {
+      if (*pit == plist->getNeutral())
+	pit = plist->begin();
+      citycount[(*pit)->getId()]++;
+      cities_left--;
+      if (cities_left == 0)
+	break;
+    }
+  for (unsigned int i = 0; i < MAX_PLAYERS; i++)
+    {
+      for (unsigned int j = 0; j < citycount[i]; j++)
+	{
+	  Player *p = plist->getPlayer(i);
+	  pos = clist->getFirstCity(p)->getPos();
+	  City *c = clist->getNearestNeutralCity(pos);
+	  c->conquer(p);
+	  History_CityWon *item = new History_CityWon();
+	  item->fillData(c);
+	  p->getHistorylist()->push_back(item);
+	}
+    }
+}
