@@ -42,9 +42,10 @@ class CreateScenario
     
         /** The Constructor
           * 
-          * @param uncle    the progress bar which displays the advance
+          * @param width    the width of the map in the new scenario
+          * @param height   the heightof the map in the new scenario
           */
-        CreateScenario();
+        CreateScenario(int width = 112, int height = 156);
         ~CreateScenario();
 
         // setters
@@ -174,7 +175,9 @@ class CreateScenario
         bool setupSignposts();
 
         //! Do some setup concerning the players (give them money)
-        bool setupPlayers();
+	//! If we're playing with diplomacy then we start out at peace,
+	//! and if we're not playing with diplomacy we start out at war.
+        bool setupPlayers(bool dimplomacy, bool random_turns);
 
         /** Loads a list of possible city, ruin or temple names
           * 
@@ -188,6 +191,7 @@ class CreateScenario
         bool setupItemRewards();
         bool setupRuinRewards();
 	bool setupMapRewards();
+	void quickStart();
 
 
         //data
