@@ -60,10 +60,13 @@ bool GameScenario::s_see_opponents_production = false;
 bool GameScenario::s_play_with_quests = true;
 bool GameScenario::s_hidden_map = false;
 bool GameScenario::s_diplomacy = false;
+bool GameScenario::s_cusp_of_war = false;
 GameParameters::NeutralCities GameScenario::s_neutral_cities = GameParameters::AVERAGE;
+GameParameters::RazingCities GameScenario::s_razing_cities = GameParameters::ALWAYS;
 bool GameScenario::s_intense_combat = false;
 bool GameScenario::s_military_advisor = false;
 bool GameScenario::s_random_turns = false;
+bool GameScenario::s_surrender_already_offered = false;
 
 GameScenario::GameScenario(std::string name,std::string comment, bool turnmode)
     :d_round(0), d_name(name),d_comment(comment), d_turnmode(turnmode)
@@ -233,10 +236,14 @@ bool GameScenario::saveGame(string filename, string extension) const
     retval &= helper.saveData("quests", s_play_with_quests);
     retval &= helper.saveData("hidden_map", s_hidden_map);
     retval &= helper.saveData("diplomacy", s_diplomacy);
+    retval &= helper.saveData("cusp_of_war", s_cusp_of_war);
     retval &= helper.saveData("neutral_cities", (int) s_neutral_cities);
+    retval &= helper.saveData("razing_cities", (int) s_razing_cities);
     retval &= helper.saveData("intense_combat", s_intense_combat);
     retval &= helper.saveData("military_advisor", s_military_advisor);
     retval &= helper.saveData("random_turns", s_random_turns);
+    retval &= helper.saveData("surrender_already_offered", 
+			      s_surrender_already_offered);
     
     retval &= helper.closeTag();
     
@@ -279,12 +286,18 @@ bool GameScenario::load(std::string tag, XML_Helper* helper)
         helper->getData(s_play_with_quests, "quests");
         helper->getData(s_hidden_map, "hidden_map");
         helper->getData(s_diplomacy, "diplomacy");
+        helper->getData(s_cusp_of_war, "cusp_of_war");
         int val = -1;
         helper->getData(val, "neutral_cities");
         s_neutral_cities = GameParameters::NeutralCities (val);
+        val = -1;
+        helper->getData(val, "razing_cities");
+        s_razing_cities = GameParameters::RazingCities (val);
         helper->getData(s_intense_combat, "intense_combat");
         helper->getData(s_military_advisor, "military_advisor");
         helper->getData(s_random_turns, "random_turns");
+        helper->getData(s_surrender_already_offered, 
+			"surrender_already_offered");
 
         //TODO: for later when it becomes crucial: deal with loading of
         //something else than simple games
