@@ -54,7 +54,10 @@ QuestCityOccupy::QuestCityOccupy (QuestsManager& q_mgr, XML_Helper* helper)
 //=======================================================================
 bool QuestCityOccupy::isFeasible(Uint32 heroId)
 {
+  if (QuestCityOccupy::chooseToOccupy(getHeroById(heroId)->getPlayer()))
     return true;
+
+  return false;
 }
 //=======================================================================
 bool QuestCityOccupy::save(XML_Helper* helper) const
@@ -105,8 +108,8 @@ void QuestCityOccupy::initDescription()
   const City* c = getCity();
   char buffer[121]; buffer[120]='\0';
 
-  snprintf(buffer, 100, _("You must occupy the city \"%s\" located at (%i,%i)."),
-	   c->getName().c_str(), c->getPos().x, c->getPos().y);
+  snprintf(buffer, 100, _("You must take over the city \"%s\" and occupy it."),
+	   c->getName().c_str());
 
   d_description = std::string(buffer);
 }
@@ -118,7 +121,8 @@ City * QuestCityOccupy::chooseToOccupy(Player *p)
   // Collect all cities
   Citylist* cl = Citylist::getInstance();
   for (Citylist::iterator it = cl->begin(); it != cl->end(); ++it)
-    if (!(*it).isBurnt() && (*it).getPlayer() != p)
+    if (!(*it).isBurnt() && (*it).getPlayer() != p &&
+	(*it).getPlayer() != Playerlist::getInstance()->getNeutral())
       cities.push_back(&(*it));
 
   // Find a suitable city for us to occupy
