@@ -54,7 +54,10 @@ QuestCityRaze::QuestCityRaze (QuestsManager& q_mgr, XML_Helper* helper)
 //=======================================================================
 bool QuestCityRaze::isFeasible(Uint32 heroId)
 {
+  if (QuestCityRaze::chooseToRaze(getHeroById(heroId)->getPlayer()))
     return true;
+
+  return false;
 }
 //=======================================================================
 bool QuestCityRaze::save(XML_Helper* helper) const
@@ -104,8 +107,9 @@ void QuestCityRaze::initDescription()
     const City* c = getCity();
     char buffer[121]; buffer[120]='\0';
     
-    snprintf(buffer, 100, _("You must raze the city \"%s\" located at (%i,%i)."),
-            c->getName().c_str(), c->getPos().x, c->getPos().y);
+    snprintf(buffer, 100, 
+	     _("You must conquer the city \"%s\" and burn it to the ground."),
+            c->getName().c_str());
 
     d_description = std::string(buffer);
 }
@@ -117,7 +121,8 @@ City* QuestCityRaze::chooseToRaze(Player *p)
     // Collect all cities
     Citylist* cl = Citylist::getInstance();
     for (Citylist::iterator it = cl->begin(); it != cl->end(); ++it)
-        if (!(*it).isBurnt() && (*it).getPlayer() != p)
+        if (!(*it).isBurnt() && (*it).getPlayer() != p &&
+	    (*it).getPlayer() != Playerlist::getInstance()->getNeutral())
             cities.push_back(&(*it));
 
     // Find a suitable city for us to raze 
