@@ -28,6 +28,7 @@
 #include "rewardlist.h"
 #include "army.h"
 #include "xmlhelper.h"
+#include "history.h"
 
 QuestsManager* QuestsManager::s_instance = NULL;
 
@@ -82,7 +83,7 @@ QuestsManager::~QuestsManager()
     _cleanup();
 }
 //======================================================================
-Quest* QuestsManager::createNewQuest(Uint32 heroId)
+Quest* QuestsManager::createNewQuest(Uint32 heroId, bool razing_possible)
 {
     // don't let a hero have more than one quest
     if (d_quests.count(heroId))
@@ -112,7 +113,10 @@ Quest* QuestsManager::createNewQuest(Uint32 heroId)
             quest = new QuestCitySack(*this, heroId);
             break;
         case 4:
-            quest = new QuestCityRaze(*this, heroId);
+	    if (razing_possible)
+	      quest = new QuestCityRaze(*this, heroId);
+	    else
+	      quest = new QuestCitySack(*this, heroId);
             break;
         case 5:
             quest = new QuestCityOccupy(*this, heroId);
@@ -155,6 +159,10 @@ void QuestsManager::questCompleted(Uint32 heroId)
         Reward_Allies reward(a, num);
         p->giveReward(p->getActivestack(), &reward);
         quest_completed.emit(quest, &reward);
+	    
+	History_HeroFindsAllies* item = new History_HeroFindsAllies();
+	item->fillData(quest->getHero());
+	p->getHistorylist()->push_back(item);
       }
     else if (num == 2)
       {
