@@ -28,27 +28,28 @@
 #include "Threatlist.h"
 #include "action.h"
 #include "xmlhelper.h"
-#include "history.h"
+#include "AI_Diplomacy.h"
 
 using namespace std;
 
 #define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
 //#define debug(x)
 
-AI_Fast::AI_Fast(string name, Uint32 armyset, SDL_Color color, int player_no)
-    :RealPlayer(name, armyset, color, Player::AI_FAST, player_no), d_join(true),
-    d_maniac(false), d_analysis(0)
+AI_Fast::AI_Fast(string name, Uint32 armyset, SDL_Color color, int width, int height, int player_no)
+    :RealPlayer(name, armyset, color, width, height, Player::AI_FAST, player_no), d_join(true),
+    d_maniac(false), d_analysis(0), d_diplomacy(0)
 {
 }
 
 AI_Fast::AI_Fast(const Player& player)
-    :RealPlayer(player), d_join(true), d_maniac(false), d_analysis(0)
+    :RealPlayer(player), d_join(true), d_maniac(false), d_analysis(0), 
+    d_diplomacy(0)
 {
     d_type = AI_FAST;
 }
 
 AI_Fast::AI_Fast(XML_Helper* helper)
-    :RealPlayer(helper), d_analysis(0)
+    :RealPlayer(helper), d_analysis(0), d_diplomacy(0)
 {
     helper->getData(d_join, "join");
     helper->getData(d_maniac, "maniac");
@@ -58,6 +59,8 @@ AI_Fast::~AI_Fast()
 {
     if (d_analysis)
         delete d_analysis;
+    if (d_diplomacy)
+        delete d_diplomacy;
 }
 
 bool AI_Fast::save(XML_Helper* helper) const
@@ -82,6 +85,7 @@ bool AI_Fast::startTurn()
     RealPlayer::startTurn();
 
     d_analysis = new AI_Analysis(this);
+    d_diplomacy = new AI_Diplomacy(this);
     
     // this is a recursively-programmed quite staightforward AI,we just call:
     computerTurn();
@@ -91,6 +95,8 @@ bool AI_Fast::startTurn()
 
     d_stacklist->setActivestack(0);
 
+    // Declare war with enemies, make peace with friends
+    d_diplomacy->makeProposals();
     return true;
 }
 
@@ -120,10 +126,6 @@ bool AI_Fast::recruitHero(Hero* hero, City *city, int cost)
 {
     debug("AI_Fast: hero offers service")
 
-    History_HeroEmerges *item = new History_HeroEmerges();
-    item->fillData(hero, city);
-    d_history.push_back(item);
-
     // maniac AI's never recruit heroes, otherwise take everything we can get
     if (d_maniac)
         return false;
@@ -169,7 +171,7 @@ void AI_Fast::computerTurn()
     //      attack
     //
 
-    // we are usign reversed order because new stacks come behind old stacks
+    // we are using reversed order because new stacks come behind old stacks
     // and we want the freshly created stacks join the veterans and not the other
     // way round.
     for (Stacklist::reverse_iterator it = d_stacklist->rbegin(); it != d_stacklist->rend(); it++)
@@ -208,7 +210,7 @@ void AI_Fast::computerTurn()
             {
                 debug("Joining with stack " <<target->getId() <<" at (" <<target->getPos().x
                       <<"," <<target->getPos().y <<")")
-                s->getPath()->calculate(s, target->getPos());
+		s->getPath()->calculate(s, target->getPos());
                 stackMove(s);
 		continue;
             }
@@ -251,13 +253,48 @@ void AI_Fast::computerTurn()
             // third step: non-maniac players attack only enemy cities
             else
             {
-                target = Citylist::getInstance()->getNearestEnemyCity(s->getPos());
+	        City *target1;
+	        City *target2;
+		Path *target1_path = new Path();
+		Path *target2_path = new Path();
+	        Citylist *cl = Citylist::getInstance();
+                target1 = cl->getNearestEnemyCity(s->getPos());
+		target2 = cl->getNearestForeignCity(s->getPos());
+		if (target1)
+		  target1_path->calculate (s, target1->getPos());
+		target2_path->calculate (s, target2->getPos());
+		if (!target1)
+		  target = target2;
+		else if (target1_path->size() / 13 > target2_path->size())
+		  target = target2;
+		else
+		  target = target1;
+
+		if (target == target2)
+		  {
+		    d_diplomacy->needNewEnemy(target->getPlayer());
+		    // try to wait a turn until we're at war
+		    if (target1)
+		      target = target1;
+		  }
+
                 if (!target)    // strange situation
                     return;
 
                 debug("Attacking " <<target->getName())
                 s->getPath()->calculate(s, target->getPos());
-                stackMove(s);
+		if (s->getPath()->checkPath(s) == true)
+		  stackMove(s);
+		else
+		  {
+		    Citylist *cl = Citylist::getInstance();
+		    City *friendly_city = 
+		      cl->getNearestFriendlyCity(s->getPos());
+		    s->getPath()->calculate(s, friendly_city->getPos());
+		    stackMove(s);
+		    //we should *always* be able to reach a target city!
+		    //this is an error in map generation.
+		  }
 
                 // a stack has died ->restart
                 if (!d_stacklist->getActivestack())
@@ -313,12 +350,23 @@ void AI_Fast::computerTurn()
             }
             else
             {
-                pos  = Citylist::getInstance()->getNearestEnemyCity(s->getPos())->getPos();
+                pos  = Citylist::getInstance()->getNearestForeignCity(s->getPos())->getPos();
                 debug("Maniac, found no targets, attacking city at (" <<pos.x <<"," <<pos.y <<")")
             }
 
             s->getPath()->calculate(s, pos);
-            stackMove(s);
+	    if (s->getPath()->checkPath(s) == true)
+	      stackMove(s);
+	    else
+	      {
+		Citylist *cl = Citylist::getInstance();
+		City *friendly_city = cl->getNearestFriendlyCity(s->getPos());
+		s->getPath()->calculate(s, friendly_city->getPos());
+		stackMove(s);
+		//FIXME: again, we should be able to reach any city,
+		//but we can't reach this one.  just go to the nearest city
+		//instead.  this is an error in map generation
+	      }
 
             if (!d_stacklist->getActivestack())
             {
@@ -329,4 +377,9 @@ void AI_Fast::computerTurn()
     }
 }
 
+bool AI_Fast::treachery (Stack *stack, Player *player, Vector <int> pos, DiplomaticState state)
+{
+  bool performTreachery = true;
+  return performTreachery;
+}
 // End of file
