@@ -316,19 +316,22 @@ void City::removeBasicProd(int index)
 
 void City::conquer(Player* newowner)
 {
+  Citylist::getInstance()->stopVectoringTo(this);
     d_player = newowner;
 
     // remove vectoring info 
     setVectoring(Vector<int>(-1,-1));
 
+    for (int j = 0; j < getNoOfBasicProd(); j++)
+      d_basicprod[j]->setPlayer(newowner);
+
     deFog(newowner);
 
     VectoredUnitlist *vul = VectoredUnitlist::getInstance();
-    vul->removeVectoredUnitsComingFrom(d_pos);
-    vul->removeVectoredUnitsGoingTo(d_pos);
+    vul->removeVectoredUnitsGoingTo(this);
+    vul->removeVectoredUnitsComingFrom(this);
 }
 
-
 void City::randomlyImproveOrDegradeArmy(Army *army)
 {
   if (rand() % 30 == 0) //random chance of improving strength
