@@ -58,9 +58,13 @@ class Citylist : public ObjectList<City>, public sigc::trackable
         //! Count the number of cities of player p.
         int countCities(Player* p) const;
 
-        //! Returns the city closest to pos that isn't owned by the ActivePlayer
+        //! Returns the city closest to pos that is owned by a player that
+	//! Activeplayer is at war with.
         City* getNearestEnemyCity(const Vector<int>& pos);
 
+        //! Returns the city closest to pos that isn't owned by the ActivePlayer
+	City* getNearestForeignCity(const Vector<int>& pos);
+
         //! Returns the city closest to pos that is owned by the ActivePlayer
         City* getNearestFriendlyCity(const Vector<int>& pos);
 
@@ -74,6 +78,9 @@ class Citylist : public ObjectList<City>, public sigc::trackable
         //! Returns the city closest to pos
         City* getNearestCity(const Vector<int>& pos);
 
+	//! returns the city closest to pos that is owned by P
+	City* getNearestCity(const Vector<int>& pos, Player *p);
+
         //! Returns the city closest to pos and is owned by the Neutrals
         City* getNearestNeutralCity(const Vector<int>& pos);
 
@@ -99,6 +106,12 @@ class Citylist : public ObjectList<City>, public sigc::trackable
 	//! owner.
 	void changeOwnership(Player *old_owner, Player *new_owner);
 
+	//! stops vectoring from any city to the specified city.
+	void stopVectoringTo(City *c);
+
+	//! find the center point for all of the given player's cities
+	Vector<int> calculateCenterOfTerritory (Player *p);
+
     protected:
         // CREATORS
         Citylist();
