@@ -60,8 +60,13 @@ struct GameParameters
         AVERAGE = 0, STRONG, ACTIVE
     };
     NeutralCities neutral_cities;
+    enum RazingCities {
+        NEVER = 0, ON_CAPTURE, ALWAYS
+    };
+    RazingCities razing_cities;
 
     bool quick_start;
+    bool cusp_of_war;
     bool intense_combat;
     bool military_advisor;
     bool random_turns;
