@@ -35,6 +35,7 @@
 #include "smallmap.h"
 
 #include "army.h"
+#include "fight.h"
 #include "hero.h"
 #include "stacklist.h"
 #include "citylist.h"
@@ -56,6 +57,7 @@
 #include "game-parameters.h"
 #include "FogMap.h"
 #include "GameMap.h"
+#include "history.h"
 
 
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<flush<<endl;}
@@ -85,6 +87,12 @@ void Game::addPlayer(Player *p)
 	  (sigc::mem_fun
 	   (hero_offers_service, &sigc::signal<bool, Player *, 
 	    Hero *, City *, int>::emit), p)));
+      connections[p->getId()].push_back
+	(p->streachery.connect
+	 (sigc::bind<0>
+	  (sigc::mem_fun
+	   (stack_considers_treachery, 
+	    &sigc::signal<bool, Player *, Stack *, Player *, Vector<int> >::emit), p)));
     }
 	
       
@@ -96,6 +104,8 @@ void Game::addPlayer(Player *p)
     (p->supdatingStack.connect (sigc::mem_fun(this, &Game::stackUpdate)));
   connections[p->getId()].push_back
     (p->sinvadingCity.connect(sigc::mem_fun(this, &Game::invading_city)));
+  connections[p->getId()].push_back
+    (p->streacheryStack.connect(sigc::mem_fun(this, &Game::maybeTreachery)));
   connections[p->getId()].push_back
     (p->fight_started.connect (sigc::mem_fun(*this, &Game::on_fight_started)));
   connections[p->getId()].push_back
@@ -106,6 +116,8 @@ void Game::addPlayer(Player *p)
     (p->ruinfight_finished.connect
      (sigc::mem_fun
       (ruinfight_finished, &sigc::signal<void, Fight::Result>::emit)));
+  connections[p->getId()].push_back
+    (p->cityfight_finished.connect (sigc::mem_fun(*this, &Game::on_city_fight_finished))); 
   connections[p->getId()].push_back
     (p->advice_asked.connect
      (sigc::mem_fun(advice_asked, &sigc::signal<void, float>::emit)));
@@ -117,7 +129,11 @@ Game::Game(GameScenario* gameScenario)
     input_locked = false;
     
     // init the bigmap
-    bigmap.reset(new GameBigMap);
+    bigmap.reset(new GameBigMap
+		 (GameScenario::s_intense_combat, 
+		  GameScenario::s_see_opponents_production, 
+		  GameScenario::s_see_opponents_stacks, 
+		  GameScenario::s_military_advisor));
     bigmap->stack_selected.connect(
 	sigc::mem_fun(this, &Game::on_stack_selected));
     bigmap->path_set.connect(
@@ -158,13 +174,17 @@ Game::Game(GameScenario* gameScenario)
 	addPlayer(p);
     }
     pl->splayerDead.connect(sigc::mem_fun(this, &Game::on_player_died));
+    pl->ssurrender.connect(sigc::mem_fun(this, &Game::on_surrender_offered));
 
     //set up a NextTurn object
-    d_nextTurn = new NextTurn(d_gameScenario->getTurnmode());
+    d_nextTurn = new NextTurn(d_gameScenario->getTurnmode(),
+			      d_gameScenario->s_random_turns);
     d_nextTurn->splayerStart.connect(
 	sigc::mem_fun(this, &Game::init_turn_for_player));
     d_nextTurn->snextRound.connect(
 	sigc::mem_fun(d_gameScenario, &GameScenario::nextRound));
+    d_nextTurn->snextRound.connect(
+	sigc::mem_fun(this, &Game::nextRound));
     d_nextTurn->supdating.connect(
 	sigc::mem_fun(this, &Game::redraw));
             
@@ -216,6 +236,8 @@ void Game::update_sidebar_stats()
 {
     SidebarStats s;
     Player *player = Playerlist::getActiveplayer();
+    if (player == Playerlist::getInstance()->getNeutral())
+      return;
 
     s.name = player->getName();
     s.gold = player->getGold();
@@ -259,9 +281,75 @@ void Game::select_next_movable_stack()
   select_active_stack();
 }
 
-void Game::move_selected_stack()
+void Game::move_selected_stack_dir(int diffx, int diffy)
 {
   Stack *stack = Playerlist::getActiveplayer()->getActivestack();
+  // Get rid of the old path if there is one
+  if (stack->getPath()->size())
+    stack->getPath()->flClear();
+  //See if we can move there
+  Vector<int> dest = stack->getPos();
+  dest.x += diffx;
+  dest.y += diffy;
+  if (stack->getPath()->canMoveThere(stack, dest))
+    {
+      // Set in a new path
+      stack->getPath()->calculate(stack, dest);
+      move_selected_stack_along_path();
+    }
+  else
+    {
+      Playerlist::getActiveplayer()->getStacklist()->setActivestack(0);
+      unselect_active_stack();
+    }
+}
+
+void Game::move_selected_stack_northwest()
+{
+  move_selected_stack_dir(-1, -1);
+}
+
+void Game::move_selected_stack_north()
+{
+  move_selected_stack_dir(0, -1);
+}
+
+void Game::move_selected_stack_northeast()
+{
+  move_selected_stack_dir(1, -1);
+}
+
+void Game::move_selected_stack_east()
+{
+  move_selected_stack_dir(1, 0);
+}
+
+void Game::move_selected_stack_west()
+{
+  move_selected_stack_dir(-1, 0);
+}
+
+void Game::move_selected_stack_southwest()
+{
+  move_selected_stack_dir(-1, 1);
+}
+
+void Game::move_selected_stack_south()
+{
+  move_selected_stack_dir(0, 1);
+}
+
+void Game::move_selected_stack_southeast()
+{
+  move_selected_stack_dir(1, 1);
+}
+
+void Game::move_selected_stack_along_path()
+{
+  Stack *stack = Playerlist::getActiveplayer()->getActivestack();
+  if (stack->isGrouped() == false)
+    Playerlist::getActiveplayer()->stackSplit(stack);
+
   Playerlist::getActiveplayer()->stackMove(stack);
 		  
   if (stack->canMove() == false)
@@ -283,6 +371,8 @@ void Game::move_all_stacks()
 	{
 	  sl->setActivestack(&s);
 	  select_active_stack();
+	  if (player->getActivestack()->isGrouped() == false)
+	    player->stackSplit(player->getActivestack());
 	  player->stackMove(player->getActivestack());
 	  i = sl->begin();
 	}
@@ -350,7 +440,8 @@ void Game::search_selected_stack()
   Ruin* ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
   Temple* temple = Templelist::getInstance()->getObjectAt(stack->getPos());
 
-  if (ruin && !ruin->isSearched() && stack->getGroupMoves() > 0 &&
+  if (ruin && !ruin->isSearched() && stack->hasHero() &&
+      stack->getFirstHero()->getMoves() > 0 &&
       ((ruin->isHidden() == true && ruin->getOwner() == player) ||
        ruin->isHidden() == false))
     {
@@ -367,6 +458,7 @@ void Game::search_selected_stack()
 	{
 	  player->giveReward(Playerlist::getActiveplayer()->getActivestack(),
 			     reward);
+	  //FIXME: delete this reward, but don't delete the item, or map
 	  redraw();
 	  update_stack_info();
 	  update_control_panel();
@@ -390,7 +482,9 @@ void Game::search_selected_stack()
       bool wants_quest = temple_searched.emit(stack->hasHero(), temple, blessCount);
       if (wants_quest)
 	{
-	  Quest *q = player->stackGetQuest(stack, temple);
+	  Quest *q = player->stackGetQuest
+	    (stack, temple, 
+	     GameScenario::s_razing_cities != GameParameters::NEVER);
 	  Hero* hero = dynamic_cast<Hero*>(stack->getFirstHero());
 
 	  if (q)
@@ -604,8 +698,18 @@ void Game::invading_city(City* city)
   Playerlist *plist = Playerlist::getInstance();
   Player *player = plist->getActiveplayer();
   int gold = 0;
-  // if a computer makes it's turn and occupied a city, we shouldn't
-  // show a modal dialog :)
+
+  // See if this is the last city for that player, and alter the 
+  // diplomatic scores.
+  if (Citylist::getInstance()->countCities(city->getPlayer()) == 1)
+    {
+      if (city->getPlayer()->getDiplomaticRank() < 
+	  player->getDiplomaticRank())
+	player->deteriorateDiplomaticRelationship (2);
+      else if (city->getPlayer()->getDiplomaticRank() > 
+	  player->getDiplomaticRank())
+	player->improveDiplomaticRelationship (2, city->getPlayer());
+    }
 
   // loot the city
   // if the attacked city isn't neutral, loot some gold
@@ -626,6 +730,7 @@ void Game::invading_city(City* city)
       case CITY_DEFEATED_RAZE:
 	player->cityRaze(city);
 	city_razed.emit (city);
+	player->deteriorateDiplomaticRelationship (5);
 	break;
 
       case CITY_DEFEATED_PILLAGE:
@@ -682,6 +787,7 @@ void Game::update_control_panel()
       can_inspect_selected_stack.emit(false);
       can_plant_standard_selected_stack.emit(false);
       can_move_selected_stack.emit(false);
+      can_move_selected_stack_along_path.emit(false);
       can_move_all_stacks.emit(false);
       can_group_ungroup_selected_stack.emit(false);
       can_end_turn.emit(false);
@@ -726,29 +832,31 @@ void Game::update_control_panel()
 
   if (stack)
     {
-      can_move_selected_stack.emit(stack->getPath()->size() > 0
-				   && stack->enoughMoves());
+      can_move_selected_stack_along_path.emit
+	(stack->getPath()->size() > 0 && stack->enoughMoves() ||
+	 (stack->getPath()->size() && stack->getMovesExhaustedAtPoint() > 0));
 
+      /*
+       * a note about searching.
+       * ruins can be searched by stacks that have a hero, and when the
+       * hero has moves left.  also the ruin must be unexplored.
+       * temples can be searched by any stack, when the stack has 
+       * movement left.
+       */
       if (stack->getGroupMoves() > 0)
 	{
 	  Temple *temple;
 	  temple = Templelist::getInstance()->getObjectAt(stack->getPos());
-	  if (stack->hasHero())
-	    {
-	      Ruin *ruin
-		= Ruinlist::getInstance()->getObjectAt(stack->getPos());
-
-	      can_search_selected_stack.emit(
-					     (ruin && !ruin->isSearched()) || temple);
-	    }
-	  else
-	    {
-	      can_search_selected_stack.emit(temple);
-	    }
+	  can_search_selected_stack.emit(temple);
+	  can_move_selected_stack.emit(true);
 	}
 
       if (stack->hasHero())
 	{
+	  Ruin *ruin = Ruinlist::getInstance()->getObjectAt(stack->getPos());
+	  if (stack->getFirstHero()->getMoves() > 0 && ruin)
+	    can_search_selected_stack.emit(!ruin->isSearched());
+
 	  can_inspect_selected_stack.emit(true);
 	  //does the hero have the player's standard?
 	  for (Stack::iterator it = stack->begin(); it != stack->end(); it++)
@@ -795,6 +903,11 @@ void Game::update_control_panel()
 		}
 	    }
 	}
+      else
+	{
+	  can_inspect_selected_stack.emit(false);
+	  can_plant_standard_selected_stack.emit(false);
+	}
 
       if (Signpostlist::getInstance()->getObjectAt(stack->getPos()))
 	can_change_signpost.emit(true);
@@ -806,6 +919,7 @@ void Game::update_control_panel()
     {
       can_search_selected_stack.emit(false);
       can_move_selected_stack.emit(false);
+      can_move_selected_stack_along_path.emit(false);
       can_disband_stack.emit(false);
       can_group_ungroup_selected_stack.emit(false);
       can_inspect_selected_stack.emit(false);
@@ -899,6 +1013,8 @@ void Game::maybeRecruitHero (Player *p)
   City *city;
   Playerlist *plist = Playerlist::getInstance();
   int gold_needed = 0;
+  if (Citylist::getInstance()->countCities(p) == 0)
+    return;
   //give the player a hero if it's the first round.
   //otherwise we get a hero based on chance
   //a hero costs a random number of gold pieces
@@ -943,6 +1059,10 @@ void Game::maybeRecruitHero (Player *p)
       bool accepted = p->recruitHero(newhero, city, gold_needed);
       if (accepted)
 	{
+	  History_HeroEmerges *item = new History_HeroEmerges();
+	  item->fillData(newhero, city);
+	  p->getHistorylist()->push_back(item);
+
 	  newhero->setPlayer(p);
 
 	  int alliesCount;
@@ -969,6 +1089,7 @@ void Game::maybeRecruitHero (Player *p)
 	    }
 	  if (gold_needed == 0)
 	    {
+	      // Initially give the first hero the player's standard.
 	      std::string name = p->getName() + " " + _("Standard");
 	      Item *battle_standard = new Item (name, true, p);
 	      battle_standard->setBonus(Item::ADD1STACK);
@@ -1052,11 +1173,15 @@ Game::loadHeroTemplates()
 
 bool Game::init_turn_for_player(Player* p)
 {
+  Playerlist* pl = Playerlist::getInstance();
   // FIXME: Currently this function only checks for a human player. You
   // can also have it check for e.g. escape key pressed to interrupt
   // an AI-only game to save/quit.
 
   center_view_on_city();
+  if (Playerlist::isFinished())
+    return true; //closing game window while the computer is moving
+
   next_player_turn.emit(p, d_gameScenario->getRound() + 1);
   if (p->getType() == Player::HUMAN)
     {
@@ -1077,6 +1202,23 @@ bool Game::init_turn_for_player(Player* p)
 	  city_visited.emit(clist->getFirstCity(p));
 	}
 
+      // update the diplomacy icon if we've received a proposal
+      bool proposal_received = false;
+      for (Playerlist::iterator it = pl->begin(); it != pl->end(); ++it)
+	{
+	  if ((*it) == pl->getNeutral())
+	    continue;
+	  if ((*it) == p)
+	    continue;
+	  if((*it)->isDead())
+	    continue;
+	  if ((*it)->getDiplomaticProposal(p) != Player::NO_PROPOSAL)
+	    {
+	      proposal_received = true;
+	      break;
+	    }
+	}
+      received_diplomatic_proposal.emit(proposal_received);
 
       return true;
     }
@@ -1085,7 +1227,27 @@ bool Game::init_turn_for_player(Player* p)
       SDL_Delay(250);
       QuestsManager::getInstance()->nextTurn(p);
       maybeRecruitHero(p);
-      return false;
+      if (d_gameScenario->s_cusp_of_war == true &&
+	  d_gameScenario->getRound() == CUSP_OF_WAR_ROUND)
+	{
+	  for (Playerlist::iterator it = pl->begin(); it != pl->end(); ++it)
+	    if ((*it)->getType() == Player::HUMAN)
+	      {
+		if ((*it)->isDead())
+		  continue;
+		if (p->getDiplomaticState(*it) != Player::AT_WAR)
+		  {
+		    p->proposeDiplomacy (Player::PROPOSE_WAR, *it);
+		    (*it)->proposeDiplomacy (Player::PROPOSE_WAR, p);
+		    p->declareDiplomacy (Player::AT_WAR, *it);
+  
+		    History_DiplomacyWar *item1 = new History_DiplomacyWar();
+		    item1->fillData(*it);
+		    p->getHistorylist()->push_back(item1);
+		  }
+	      }
+	}
+	  return false;
     }
 }
 
@@ -1100,10 +1262,24 @@ void Game::on_player_died(Player *player)
 
 void Game::on_fight_started(Fight &fight)
 {
-  if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
-      GameScenario::s_hidden_map == true)
-    return;
-  fight_started.emit(fight);
+  Player* pd = (*(fight.getDefenders().begin()))->getPlayer();
+  Player* pa = (*(fight.getAttackers().begin()))->getPlayer();
+  if ((pa->getType() == Player::HUMAN || pd->getType() == Player::HUMAN) ||
+      (pa->getType() != Player::HUMAN && pd->getType() != Player::HUMAN 
+       && pd != Playerlist::getInstance()->getNeutral()))
+    {
+
+      if (Playerlist::getActiveplayer()->getType() != Player::HUMAN &&
+	  GameScenario::s_hidden_map == true)
+	return;
+    }
+  else
+    {
+      //short circuit the battle sequence
+      fight.battle(GameScenario::s_intense_combat);
+      return;
+    }
+  fight_started.emit(fight, GameScenario::s_intense_combat);
 }
 
 void Game::center_view_on_city()
@@ -1129,6 +1305,8 @@ void Game::center_view_on_city()
     {
       if (i->getPlayer() == p)
 	{
+	  if (Playerlist::isFinished())
+	    return;
 	  smallmap->center_view(i->getPos(), true);
 	  break;
 	}
@@ -1144,3 +1322,88 @@ void Game::unselect_active_stack()
 {
   bigmap->unselect_active_stack();
 }
+
+bool Game::maybeTreachery(Stack *stack, Player *them, Vector<int> pos)
+{
+  Player *me = stack->getPlayer();
+  bool treachery = me->treachery (stack, them, pos);
+  if (treachery == false)
+    return false;
+  me->proposeDiplomacy (Player::NO_PROPOSAL, them);
+  me->declareDiplomacy (Player::AT_WAR, them);
+  them->proposeDiplomacy (Player::NO_PROPOSAL, me);
+  them->declareDiplomacy (Player::AT_WAR, me);
+  History_DiplomacyTreachery *item = new History_DiplomacyTreachery();
+  item->fillData(them);
+  me->getHistorylist()->push_back(item);
+
+  me->deteriorateDiplomaticRelationship (5);
+  them->improveDiplomaticRelationship (2, me);
+
+  return true;
+}
+
+void Game::nextRound()
+{
+  Playerlist::getInstance()->nextRound
+    (GameScenario::s_diplomacy, &GameScenario::s_surrender_already_offered);
+}
+    
+void Game::on_surrender_offered(Player *recipient)
+{
+  Playerlist *plist = Playerlist::getInstance();
+  if (enemy_offers_surrender.emit(plist->countPlayersAlive() - 1))
+    {
+      surrender_answered.emit(true);
+      game_over.emit(recipient);
+    }
+  else
+    surrender_answered.emit(false);
+}
+
+void Game::recalculate_moves_for_stack(Stack *s)
+{
+  if (!s)
+    s = Playerlist::getActiveplayer()->getActivestack();
+  if (s)
+    {
+      s->getPath()->recalculate(s);
+      redraw();
+      update_control_panel();
+    }
+}
+    
+void Game::on_city_fight_finished(City *city, Fight::Result result)
+{
+  if (result != Fight::ATTACKER_WON)
+    {
+      // we didn't suceed in defeating the defenders
+      //if this is a neutral city, and we're playing with 
+      //active neutral cities, AND it hasn't already been attacked
+      //then it's production gets turned on
+      Player *neu = city->getPlayer(); //neutral player
+      if (GameScenario::s_neutral_cities == GameParameters::ACTIVE &&
+	  neu == Playerlist::getInstance()->getNeutral() &&
+	  city->getProductionIndex() == -1)
+	{
+	  //great, then let's turn on the production.
+	  //well, we already made a unit, and we want to produce more
+	  //of it.
+	  Stack *o = neu->getStacklist()->getObjectAt(city->getPos());
+	  if (o)
+	    {
+	      int army_type = o->getStrongestArmy()->getType();
+	      for (int i = 0; i < 4; i++)
+		{
+		  if (city->getArmytype(i) == army_type)
+		    {
+		      // hey, we found the droid we were looking for
+		      city->setProduction(i);
+		      break;
+		    }
+		}
+	    }
+	}
+    }
+  return;
+}
