@@ -37,7 +37,6 @@
 #include "defs.h"
 #include "File.h"
 #include "GameMap.h"
-#include "GameScenario.h"
 #include "GraphicsCache.h"
 #include "game.h"
 
@@ -54,8 +53,14 @@ namespace
     int selection_timeout = 150;	// controls speed of selector rotation
 }
 
-GameBigMap::GameBigMap()
+GameBigMap::GameBigMap(bool intense_combat, bool see_opponents_production,
+		       bool see_opponents_stacks, bool military_advisor)
 {
+  d_intense_combat = intense_combat;
+  d_see_opponents_production = see_opponents_production;
+  d_see_opponents_stacks = see_opponents_stacks;
+  d_military_advisor = military_advisor;
+
   current_tile.x = current_tile.y = 0;
   mouse_state = NONE;
   input_locked = false;
@@ -120,10 +125,12 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 
       if (stack)
 	{
+	  bool path_already_set = stack->getPath()->size() > 0;
 	  // ask for military advice
 	  if (d_cursor == GraphicsCache::QUESTION)
 	    {
-	      Playerlist::getActiveplayer()->stackFightAdvise(stack, tile); 
+	      Playerlist::getActiveplayer()->stackFightAdvise
+		(stack, tile, d_intense_combat); 
 	      set_shift_key_down (false);
 	      return;
 	    }
@@ -169,12 +176,48 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    {
 	      Playerlist::getActiveplayer()->stackMove(stack);
 	      if (!Playerlist::getActiveplayer()->getActivestack())
-		return;
+		{
+		  if (stack->canMove() == false)
+		    unselect_active_stack();
+		  return;
+		}
 	      else
 		{
 		  //grab our stack again because maybe we joined another stack
 		  stack = Playerlist::getActiveplayer()->getActivestack();
-		  if (stack->canMove() == false)
+
+		  //deslect when:
+		  //1. we've moved our stack too far and we've gone as far
+		  //   as we can on our path.
+		  //2. we've set in a second path and we've gone as far as
+		  //   we can on our path.
+		  //3. we're next to an enemy city, out of moves, and we 
+		  //   try to attack the city.
+		  //note that special care is taken to not deselect when
+		  //we've proceeded along our path and ran out of nodes
+		  //to follow, but we still have moves to make.
+		  bool deselect = false;
+		  if (path_already_set)
+		    {
+		      if (stack->getPath()->size() > 0 && 
+			  stack->enoughMoves() == false)
+			deselect = true;
+		      else if (stack->getPath()->size() > 0 &&
+			       stack->getMovesExhaustedAtPoint() == 0)
+			deselect = true;
+		    }
+		  else
+		    {
+		      if (stack->getPath()->size() > 0 && 
+			  stack->getMovesExhaustedAtPoint() == 0)
+			deselect = true;
+		      if ((d_cursor == GraphicsCache::SWORD ||
+			   d_cursor == GraphicsCache::HEART) &&
+			  stack->canMove() == false)
+			deselect = true;
+		    }
+
+		  if (deselect)
 		    {
 		      Player *player = Playerlist::getActiveplayer();
 		      player->getStacklist()->setActivestack(0);
@@ -182,8 +225,6 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		    }
 		}
 	    }
-	  //else
-	    //stack->getPath()->calculate(stack, p);
 
 	  path_set.emit();
 
@@ -204,7 +245,7 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 		{
 		  if (!c->isBurnt())
 		    {
-		      if (GameScenario::s_see_opponents_production == true)
+		      if (d_see_opponents_production == true)
 			city_queried (c, false);
 		      else
 			{
@@ -259,13 +300,13 @@ void GameBigMap::mouse_button_event(MouseButtonEvent e)
 	    }
 	  else if (Stack *st = Stacklist::getObjectAt(tile.x, tile.y))
 	    {
-	      if (GameScenario::s_see_opponents_stacks == true)
+	      if (d_see_opponents_stacks == true)
 		{
 		  stack_queried.emit(st);
 		  mouse_state = SHOWING_STACK;
 		}
 	      else if (st->getPlayer() == Playerlist::getActiveplayer() && 
-		       GameScenario::s_see_opponents_stacks == false)
+		       d_see_opponents_stacks == false)
 		{
 		  stack_queried.emit(st);
 		  mouse_state = SHOWING_STACK;
@@ -342,7 +383,10 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 			d_cursor = GraphicsCache::QUESTION;
 		      else
 			{
-			  bool friendly = false;
+			  Player *me = stack->getPlayer();
+			  Player *them = c->getPlayer();
+			  bool friendly = (me->getDiplomaticState(them) == 
+					   Player::AT_PEACE);
 			  if (friendly)
 			    d_cursor = GraphicsCache::HEART;
 			  else
@@ -352,7 +396,7 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		  else
 		    {
 		      //can i see other ppl's cities?
-		      if (GameScenario::s_see_opponents_production == true)
+		      if (d_see_opponents_production == true)
 			d_cursor = GraphicsCache::ROOK;
 		      else
 			d_cursor = GraphicsCache::HAND;
@@ -391,7 +435,10 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 		    	    d_cursor = GraphicsCache::QUESTION;
 			  else
 			    {
-			      bool friendly = false;
+			      Player *me = stack->getPlayer();
+			      Player *them = st->getPlayer();
+			      bool friendly = (me->getDiplomaticState(them) == 
+					       Player::AT_PEACE);
 			      if (friendly)
 				d_cursor = GraphicsCache::HEART;
 			      else
@@ -426,10 +473,10 @@ void GameBigMap::determine_mouse_cursor(Stack *stack, Vector<int> tile)
 	    {
 	      City *c = Citylist::getInstance()->getObjectAt(tile);
 	      if (c->isBurnt() == true)
-		d_cursor = GraphicsCache::FEET;
+		d_cursor = GraphicsCache::HAND;
 	      else if (c->getPlayer() == Playerlist::getActiveplayer())
 		d_cursor = GraphicsCache::ROOK;
-	      else if (GameScenario::s_see_opponents_production == true)
+	      else if (d_see_opponents_production == true)
 		d_cursor = GraphicsCache::ROOK;
 	    }
 	  else if (t->getBuilding() == Maptile::RUIN)
@@ -585,7 +632,7 @@ void GameBigMap::after_draw()
 
 void GameBigMap::set_shift_key_down (bool down)
 {
-  if (GameScenario::s_military_advisor == false)
+  if (d_military_advisor == false)
     return;
   static GraphicsCache::CursorType prev_cursor = GraphicsCache::HEART;
 
