@@ -40,7 +40,8 @@
 #include "../citylist.h"
 #include "../playerlist.h"
 
-CityWindow::CityWindow(City *c)
+CityWindow::CityWindow(City *c, bool razing_possible, 
+		       bool see_opponents_production)
 {
     city = c;
     
@@ -53,7 +54,8 @@ CityWindow::CityWindow(City *c)
     
     xml->get_widget("map_image", map_image);
 
-    prodmap.reset(new VectorMap(c, VectorMap::SHOW_NO_VECTORING));
+    prodmap.reset(new VectorMap(c, VectorMap::SHOW_NO_VECTORING,
+		  see_opponents_production));
     prodmap->map_changed.connect(
 	sigc::mem_fun(this, &CityWindow::on_map_changed));
 
@@ -102,6 +104,7 @@ CityWindow::CityWindow(City *c)
 	
     }
 
+    d_razing_possible = razing_possible;
     fill_in_city_info();
     fill_in_production_toggles();
 
@@ -279,7 +282,7 @@ void CityWindow::fill_in_production_info()
         if (city->getVectoring() != Vector<int>(-1, -1))
           {
             Citylist *cl = Citylist::getInstance();
-            City *dest = cl->getObjectAt(city->getVectoring());
+            City *dest = cl->getNearestFriendlyCity(city->getVectoring(), 4);
             s3 += String::ucompose(_(", then to %1"), 
                                    dest ? dest->getName() : "Standard");
           }
@@ -316,7 +319,7 @@ void CityWindow::fill_in_production_info()
         current_label->show();
         current_image->show();
         buy_button->set_sensitive(true);
-        raze_button->set_sensitive(true);
+        raze_button->set_sensitive (d_razing_possible);
         rename_button->set_sensitive(true);
         destination_button->set_sensitive(true);
         on_hold_button->set_sensitive(true);
