@@ -74,7 +74,8 @@ DestinationDialog::DestinationDialog(City *c)
       sigc::bind(sigc::mem_fun(this, &DestinationDialog::on_change_toggled),
                                change_toggle));
 
-    vectormap.reset(new VectorMap(c, VectorMap::SHOW_ORIGIN_CITY_VECTORING));
+    vectormap.reset(new VectorMap(c, VectorMap::SHOW_ORIGIN_CITY_VECTORING,
+		    false));
     vectormap->map_changed.connect(
 	sigc::mem_fun(this, &DestinationDialog::on_map_changed));
 
@@ -236,7 +237,7 @@ void DestinationDialog::fill_in_vectoring_info()
 
   //show the units that are arriving into this city
   vectored.clear();
-  vul->getVectoredUnitsGoingTo(city->getPos(), vectored);
+  vul->getVectoredUnitsGoingTo(city, vectored);
   int count = 0;
   Gtk::Image *image = next_turn_1_image;
   for (it = vectored.begin(); it != vectored.end(); it++)
