@@ -38,8 +38,9 @@
 #include "../army.h"
 #include "../GraphicsCache.h"
 
-FightWindow::FightWindow(Fight &fight)
+FightWindow::FightWindow(Fight &fight, bool intense_combat)
 {
+  bool d_intense_combat = intense_combat;
     Glib::RefPtr<Gnome::Glade::Xml> xml
 	= Gnome::Glade::Xml::create(get_glade_path() + "/fight-window.glade");
 
@@ -98,7 +99,7 @@ FightWindow::FightWindow(Fight &fight)
     xml->get_widget("attacker_shield_image", attacker_shield_image);
     attacker_shield_image->property_pixbuf()=to_pixbuf(gc->getShieldPic(2, p));
   
-    fight.battle();
+    fight.battle(d_intense_combat);
     actions = fight.getCourseOfEvents();
     d_quick = false;
 }
@@ -219,7 +220,8 @@ void FightWindow::add_army(Army *army,
     Gtk::VBox *army_box = manage(new Gtk::VBox);
 	
     // image
-    army_box->add(*manage(new Gtk::Image(to_pixbuf(army->getPixmap()))));
+    Gtk::Image *image = new Gtk::Image(to_pixbuf(army->getPixmap()));
+    army_box->add(*manage(image));
     
     // hit points graph
     Gtk::ProgressBar *progress = manage(new Gtk::ProgressBar);
@@ -251,6 +253,7 @@ void FightWindow::add_army(Army *army,
     item.army = army;
     item.hp = army->getHP();
     item.bar = progress;
+    item.image = image;
     army_items.push_back(item);
 }
 
@@ -269,8 +272,13 @@ bool FightWindow::do_round()
 		i->hp -= f.damage;
 		if (i->hp < 0)
 		    i->hp = 0;
-		i->bar->set_fraction(double(i->hp)
-				     / i->army->getStat(Army::HP));
+		double fraction = double(i->hp) / i->army->getStat(Army::HP);
+		i->bar->set_fraction(fraction);
+		if (fraction == 0.0)
+		  {
+		    i->bar->hide();
+		    //i->image->get_pixbuf()->fill(0x00000000);
+		  }
 		
 		break;
 	    }
