@@ -54,6 +54,7 @@
 #include "sage-dialog.h"
 #include "ruin-rewarded-dialog.h"
 #include "hero-offer-dialog.h"
+#include "surrender-dialog.h"
 #include "quest-report-dialog.h"
 #include "quest-assigned-dialog.h"
 #include "quest-completed-dialog.h"
@@ -66,6 +67,8 @@
 #include "history-report-dialog.h"
 #include "report-dialog.h"
 #include "triumphs-dialog.h"
+#include "diplomacy-report-dialog.h"
+#include "diplomacy-dialog.h"
 
 #include "../ucompose.hpp"
 #include "../defs.h"
@@ -77,6 +80,7 @@
 #include "../GameScenario.h"
 #include "../army.h"
 #include "../ruin.h"
+#include "../path.h"
 #include "../player.h"
 #include "../stacklist.h"
 #include "../signpostlist.h"
@@ -171,6 +175,7 @@ GameWindow::GameWindow()
     // the control panel
     xml->get_widget("next_movable_button", next_movable_button);
     xml->get_widget("center_button", center_button);
+    xml->get_widget("diplomacy_button", diplomacy_button);
     xml->get_widget("defend_button", defend_button);
     xml->get_widget("park_button", park_button);
     xml->get_widget("deselect_button", deselect_button);
@@ -178,19 +183,38 @@ GameWindow::GameWindow()
     xml->get_widget("move_button", move_button);
     xml->get_widget("move_all_button", move_all_button);
     xml->get_widget("end_turn_button", end_turn_button);
+    xml->get_widget("nw_keypad_button", nw_keypad_button);
+    xml->get_widget("n_keypad_button", n_keypad_button);
+    xml->get_widget("ne_keypad_button", ne_keypad_button);
+    xml->get_widget("e_keypad_button", e_keypad_button);
+    xml->get_widget("w_keypad_button", w_keypad_button);
+    xml->get_widget("sw_keypad_button", sw_keypad_button);
+    xml->get_widget("s_keypad_button", s_keypad_button);
+    xml->get_widget("se_keypad_button", se_keypad_button);
 
     // fill in imagery
-    std::vector<Glib::RefPtr<Gdk::Pixbuf> > button_images
-	= disassemble_row(File::getMiscFile("various/buttons.png"), 11);
-    next_movable_button->add(*manage(new Gtk::Image(button_images[2])));
-    center_button->add(*manage(new Gtk::Image(button_images[5])));
-    defend_button->add(*manage(new Gtk::Image(button_images[6])));
-    park_button->add(*manage(new Gtk::Image(button_images[1])));
-    deselect_button->add(*manage(new Gtk::Image(button_images[7])));
-    search_button->add(*manage(new Gtk::Image(button_images[9])));
-    move_button->add(*manage(new Gtk::Image(button_images[3])));
-    move_all_button->add(*manage(new Gtk::Image(button_images[4])));
-    end_turn_button->add(*manage(new Gtk::Image(button_images[10])));
+    d_button_images = disassemble_row(File::getMiscFile("various/buttons.png"), 11);
+    next_movable_button->add(*manage(new Gtk::Image(d_button_images[2])));
+    center_button->add(*manage(new Gtk::Image(d_button_images[5])));
+    diplomacy_button->add(*manage(new Gtk::Image(d_button_images[0])));
+    defend_button->add(*manage(new Gtk::Image(d_button_images[6])));
+    park_button->add(*manage(new Gtk::Image(d_button_images[1])));
+    deselect_button->add(*manage(new Gtk::Image(d_button_images[7])));
+    search_button->add(*manage(new Gtk::Image(d_button_images[9])));
+    move_button->add(*manage(new Gtk::Image(d_button_images[3])));
+    move_all_button->add(*manage(new Gtk::Image(d_button_images[4])));
+    end_turn_button->add(*manage(new Gtk::Image(d_button_images[10])));
+    
+    d_arrow_images = disassemble_row(File::getMiscFile("various/arrows.png"), 
+				     8);
+    nw_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[0])));
+    n_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[1])));
+    ne_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[2])));
+    e_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[3])));
+    w_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[4])));
+    sw_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[5])));
+    s_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[6])));
+    se_keypad_button->add(*manage(new Gtk::Image(d_arrow_images[7])));
 
     // connect callbacks for the menu
     xml->connect_clicked("load_game_menuitem",
@@ -209,6 +233,8 @@ GameWindow::GameWindow()
 			 sigc::mem_fun(*this, &GameWindow::on_gold_report_activated));
     xml->connect_clicked("winning_report_menuitem",
 			 sigc::mem_fun(*this, &GameWindow::on_winning_report_activated));
+    xml->connect_clicked("diplomacy_report_menuitem",
+			 sigc::mem_fun(*this, &GameWindow::on_diplomacy_report_activated));
     xml->connect_clicked("quests_menuitem", 
 			 sigc::mem_fun(*this, &GameWindow::on_quests_activated));
     xml->connect_clicked("fullscreen_menuitem", 
@@ -264,6 +290,7 @@ void GameWindow::show()
 {
     next_movable_button->show_all();
     center_button->show_all();
+    diplomacy_button->show_all();
     defend_button->show_all();
     park_button->show_all();
     deselect_button->show_all();
@@ -271,6 +298,14 @@ void GameWindow::show()
     move_button->show_all();
     move_all_button->show_all();
     end_turn_button->show_all();
+    nw_keypad_button->show_all();
+    n_keypad_button->show_all();
+    ne_keypad_button->show_all();
+    e_keypad_button->show_all();
+    w_keypad_button->show_all();
+    sw_keypad_button->show_all();
+    s_keypad_button->show_all();
+    se_keypad_button->show_all();
     
     sdl_container->show_all();
     window->show();
@@ -323,7 +358,7 @@ void GameWindow::init(int width, int height)
 std::string
 create_and_dump_scenario(const std::string &file, const GameParameters &g)
 {
-    CreateScenario creator;
+    CreateScenario creator (g.map.width, g.map.height);
 
     // then fill the other players
     int c = 0;
@@ -367,10 +402,6 @@ create_and_dump_scenario(const std::string &file, const GameParameters &g)
     int area = g.map.width * g.map.height;
     creator.setNoSignposts(int(area * (g.map.grass / 100.0) * 0.0030));
 
-    // tell it the dimensions
-    creator.setWidth(g.map.width);
-    creator.setHeight(g.map.height);
-
     // and tell it the turn mode
     if (g.process_armies == GameParameters::PROCESS_ARMIES_AT_PLAYERS_TURN)
         creator.setTurnmode(true);
@@ -441,6 +472,13 @@ void GameWindow::setup_signals()
   setup_button(center_button,
 	       sigc::mem_fun(game.get(), &Game::center_selected_stack),
 	       game->can_center_selected_stack);
+	       
+  connections.push_back (diplomacy_button->signal_clicked().connect
+   (sigc::mem_fun (*this, &GameWindow::on_diplomacy_button_clicked)));
+  connections.push_back 
+    (game->received_diplomatic_proposal.connect 
+     (sigc::mem_fun(*this, &GameWindow::update_diplomacy_button)));
+
   setup_button(defend_button,
 	       sigc::mem_fun(game.get(), &Game::defend_selected_stack),
 	       game->can_defend_selected_stack);
@@ -454,11 +492,35 @@ void GameWindow::setup_signals()
 	       sigc::mem_fun(game.get(), &Game::search_selected_stack),
 	       game->can_search_selected_stack);
   setup_button(move_button,
-	       sigc::mem_fun(game.get(), &Game::move_selected_stack),
-	       game->can_move_selected_stack);
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_along_path),
+	       game->can_move_selected_stack_along_path);
   setup_button(move_all_button,
 	       sigc::mem_fun(game.get(), &Game::move_all_stacks),
 	       game->can_move_all_stacks);
+  setup_button(nw_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_northwest),
+	       game->can_move_selected_stack);
+  setup_button(n_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_north),
+	       game->can_move_selected_stack);
+  setup_button(ne_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_northeast),
+	       game->can_move_selected_stack);
+  setup_button(e_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_east),
+	       game->can_move_selected_stack);
+  setup_button(w_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_west),
+	       game->can_move_selected_stack);
+  setup_button(sw_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_southwest),
+	       game->can_move_selected_stack);
+  setup_button(s_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_south),
+	       game->can_move_selected_stack);
+  setup_button(se_keypad_button,
+	       sigc::mem_fun(game.get(), &Game::move_selected_stack_southeast),
+	       game->can_move_selected_stack);
   setup_menuitem(move_all_menuitem,
 		 sigc::mem_fun(game.get(), &Game::move_all_stacks),
 		 game->can_move_all_stacks);
@@ -543,6 +605,15 @@ void GameWindow::setup_signals()
   connections.push_back
     (game->hero_offers_service.connect
      (sigc::mem_fun(*this, &GameWindow::on_hero_offers_service)));
+  connections.push_back
+    (game->enemy_offers_surrender.connect
+     (sigc::mem_fun(*this, &GameWindow::on_enemy_offers_surrender)));
+  connections.push_back
+    (game->surrender_answered.connect
+     (sigc::mem_fun(*this, &GameWindow::on_surrender_answered)));
+  connections.push_back
+    (game->stack_considers_treachery.connect
+     (sigc::mem_fun(*this, &GameWindow::on_stack_considers_treachery)));
   connections.push_back
     (game->temple_searched.connect
      (sigc::mem_fun(*this, &GameWindow::on_temple_searched)));
@@ -612,6 +683,20 @@ void GameWindow::setup_signals()
 
 }
 
+void GameWindow::update_diplomacy_button (bool proposals_present)
+{
+  if (GameScenario::s_diplomacy == false)
+    {
+      diplomacy_button->set_sensitive (false);
+      return;
+    }
+  /* switch up the image. */
+  if (proposals_present)
+    diplomacy_button->property_image() = new Gtk::Image(d_button_images[8]);
+  else
+    diplomacy_button->property_image() = new Gtk::Image(d_button_images[0]);
+}
+
 void GameWindow::setup_game(std::string file_path)
 {
   stop_game();
@@ -723,7 +808,7 @@ void GameWindow::on_sdl_surface_changed()
 
 void GameWindow::on_load_game_activated()
 {
-  return;
+  //return;
   Gtk::FileChooserDialog chooser(*window.get(), _("Choose Game to Load"));
   Gtk::FileFilter sav_filter;
   sav_filter.add_pattern("*.sav");
@@ -921,9 +1006,12 @@ void GameWindow::on_resign_activated()
 
 void GameWindow::on_preferences_activated()
 {
+  Player *current = Playerlist::getInstance()->getActiveplayer();
   PreferencesDialog d;
   d.set_parent_window(*window.get());
   d.run(game.get());
+  if (current != Playerlist::getInstance()->getActiveplayer())
+    game->end_turn();
 }
 
 void GameWindow::on_group_ungroup_activated()
@@ -1045,6 +1133,20 @@ void GameWindow::on_triumphs_activated()
   d.run();
 }
 
+void GameWindow::on_diplomacy_report_activated()
+{
+  DiplomacyReportDialog d(Playerlist::getActiveplayer());
+  d.set_parent_window(*window.get());
+  d.run();
+}
+
+void GameWindow::on_diplomacy_button_clicked()
+{
+  DiplomacyDialog d(Playerlist::getActiveplayer());
+  d.set_parent_window(*window.get());
+  d.run();
+}
+
 void GameWindow::stop_game()
 {
   Sound::getInstance()->disableBackground();
@@ -1151,12 +1253,14 @@ void GameWindow::on_message_requested(std::string msg)
 
 void GameWindow::on_army_toggled(Gtk::ToggleButton *toggle, Army *army)
 {
+  Player *p = Playerlist::getActiveplayer();
+  Stack *s = p->getStacklist()->getActivestack();
   group_ungroup_toggle->set_sensitive(false);
   army->setGrouped(toggle->get_active());
   ensure_one_army_button_active();
-  Player *p = Playerlist::getActiveplayer();
-  on_stack_info_changed(p->getStacklist()->getActivestack());
+  on_stack_info_changed(s);
   group_ungroup_toggle->set_sensitive(true);
+  game->recalculate_moves_for_stack(s);
 }
 
 void GameWindow::on_group_toggled(Gtk::ToggleButton *toggle)
@@ -1167,6 +1271,7 @@ void GameWindow::on_group_toggled(Gtk::ToggleButton *toggle)
     currently_selected_stack->ungroup();
   else
     currently_selected_stack->group();
+  game->recalculate_moves_for_stack(currently_selected_stack);
   update_army_buttons();
 }
 
@@ -1443,10 +1548,6 @@ void GameWindow::on_ruin_searched(Ruin *ruin, Stack *stack, Reward *reward)
 
   dialog->set_title(ruin->getName());
 
-  Gtk::Image *image;
-  xml->get_widget("ruin_image", image);
-  image->property_file() = File::getMiscFile("various/ruin_1.jpg");
-
   Gtk::Label *label;
   xml->get_widget("label", label);
 
@@ -1533,13 +1634,20 @@ void GameWindow::on_ruinfight_finished(Fight::Result result)
     s += _("...and is slain by it!");
   label->set_text(s);
 
+  Gtk::Image *image;
+  xml->get_widget("image", image);
+  if (result == Fight::ATTACKER_WON)
+    image->property_file() = File::getMiscFile("various/ruin_2.png");
+  else
+    image->property_file() = File::getMiscFile("various/ruin_1.png");
+
   dialog->show_all();
   dialog->run();
 }
 
-void GameWindow::on_fight_started(Fight &fight)
+void GameWindow::on_fight_started(Fight &fight, bool intense_combat)
 {
-  FightWindow d(fight);
+  FightWindow d(fight, intense_combat);
 
   d.set_parent_window(*window.get());
   d.run(&d_quick_fights);
@@ -1580,6 +1688,53 @@ bool GameWindow::on_hero_offers_service(Player *player, Hero *hero, City *city,
   return d.run();
 }
 
+bool GameWindow::on_enemy_offers_surrender(int numPlayers)
+{
+  SurrenderDialog d(numPlayers);
+  d.set_parent_window(*window.get());
+  return d.run();
+}
+
+void GameWindow::on_surrender_answered (bool accepted)
+{
+  if (accepted)
+    on_message_requested
+      (_("You graciously and benevolently accept their offer."));
+  else
+    on_message_requested
+      (_("Off with their heads!  I want it ALL!"));
+}
+
+bool GameWindow::on_stack_considers_treachery (Player *me, Stack *stack, 
+					       Player *them, Vector<int> pos)
+{
+  std::auto_ptr<Gtk::Dialog> dialog;
+
+  Glib::RefPtr<Gnome::Glade::Xml> xml
+    = Gnome::Glade::Xml::create(get_glade_path() + "/treachery-dialog.glade");
+
+  Gtk::Dialog *d;
+  xml->get_widget("dialog", d);
+  dialog.reset(d);
+  dialog->set_transient_for(*window.get());
+  Gtk::Label *label;
+  xml->get_widget("label", label);
+  Glib::ustring s;
+  s = String::ucompose(_("Are you sure you want to attack %1?"), 
+		       them->getName());
+  s += "\n";
+  s += _("Other players may not like this!");
+  label->set_text(s);
+  dialog->show_all();
+  int retval = dialog->run();
+  if (retval == Gtk::RESPONSE_DELETE_EVENT)
+    return false;
+  else if (retval)
+    return true;
+  else
+    return false;
+}
+
 
 void GameWindow::on_temple_visited(Temple *temple)
 {
@@ -1610,7 +1765,8 @@ bool GameWindow::on_temple_searched(bool hasHero, Temple *temple, int blessCount
   xml->get_widget("close_button", close_button);
   xml->get_widget("accept_button", accept_button);
 
-  if (qm->getPlayerQuests(Playerlist::getActiveplayer()).size() > 0)
+  if (qm->getPlayerQuests(Playerlist::getActiveplayer()).size() > 0 &&
+      hasHero)
     accept_button->set_sensitive(false);
 
   Glib::ustring s;
@@ -1659,10 +1815,11 @@ void GameWindow::on_quest_assigned(Hero *hero, Quest *quest)
 }
 
 static bool
-hero_has_quest_here (Stack *s, City *c, bool *sack, bool *raze, bool *occupy)
+hero_has_quest_here (Stack *s, City *c, bool *pillage, bool *sack, bool *raze, bool *occupy)
 {
   Player *p = Playerlist::getActiveplayer();
   std::vector<Quest*> questlist;
+  *pillage = false;
   *sack = false;
   *raze = false;
   *occupy = false;
@@ -1713,6 +1870,7 @@ hero_has_quest_here (Stack *s, City *c, bool *sack, bool *raze, bool *occupy)
 	    }
 	  break;
 	case Quest::PILLAGEGOLD:
+	  *pillage = true;
 	  *sack = true;
 	  break;
 	}
@@ -1725,6 +1883,7 @@ hero_has_quest_here (Stack *s, City *c, bool *sack, bool *raze, bool *occupy)
 
 CityDefeatedAction GameWindow::on_city_defeated(City *city, int gold)
 {
+  Gtk::Button *raze_button;
   std::auto_ptr<Gtk::Dialog> dialog;
   if (gold)
     on_city_looted (city, gold);
@@ -1739,7 +1898,7 @@ CityDefeatedAction GameWindow::on_city_defeated(City *city, int gold)
 
   Gtk::Image *image;
   xml->get_widget("city_image", image);
-  image->property_file() = File::getMiscFile("various/city_occupied.jpg");
+  image->property_file() = File::getMiscFile("various/city_occupied.png");
   image->show();
 
   Gtk::Label *label;
@@ -1769,29 +1928,37 @@ CityDefeatedAction GameWindow::on_city_defeated(City *city, int gold)
   s += label->get_text();
   label->set_text(s);
 
+  xml->get_widget("raze_button", raze_button);
+
+  raze_button->set_sensitive
+    (game->getScenario()->s_razing_cities == GameParameters::ON_CAPTURE || 
+     game->getScenario()->s_razing_cities == GameParameters::ALWAYS);
+
   if (h) /* if there was a hero in the stack */
     {
-      bool sack, raze, occupy;
+      bool pillage, sack, raze, occupy;
       if (hero_has_quest_here (p->getStacklist()->getActivestack(), city, 
-			       &sack, &raze, &occupy))
+			       &pillage, &sack, &raze, &occupy))
 	{
 	  Gtk::Button *button;
+	  if (pillage)
+	    {
+	      xml->get_widget("pillage_button", button);
+	      button->grab_default();
+	    }
 	  if (sack)
 	    {
 	      xml->get_widget("sack_button", button);
 	      button->grab_default();
-	      //button->set_label(">" + button->get_label() +"<");
 	    }
 	  if (raze)
 	    {
 	      xml->get_widget("raze_button", button);
-	      //button->set_label(">" + button->get_label() +"<");
 	      button->grab_default();
 	    }
 	  if (occupy)
 	    {
 	      xml->get_widget("occupy_button", button);
-	      //button->set_label(">" + button->get_label() +"<");
 	      button->grab_default();
 	    }
 	}
@@ -2025,7 +2192,9 @@ void GameWindow::on_city_razed (City *city)
 
 void GameWindow::on_city_visited(City *city)
 {
-  CityWindow d(city);
+  CityWindow d(city, 
+	       game->getScenario()->s_razing_cities == GameParameters::ALWAYS,
+	       GameScenario::s_see_opponents_production);
 
   d.set_parent_window(*window.get());
   d.run();
@@ -2071,6 +2240,8 @@ void GameWindow::on_next_player_turn(Player *player, unsigned int turn_number)
 
   while (g_main_context_iteration(NULL, FALSE)); //doEvents
 
+  if (Playerlist::isFinished()) //closed window while ai player moves
+    return;
   d_quick_fights = false;
   show_shield_turn();
   if (player->getType() != Player::HUMAN)
@@ -2131,7 +2302,7 @@ void GameWindow::on_medal_awarded_to_army(Army *army)
 
 Army::Stat GameWindow::on_army_gains_level(Army *army)
 {
-  ArmyGainsLevelDialog d(army);
+  ArmyGainsLevelDialog d(army, GameScenario::s_hidden_map);
 
   d.set_parent_window(*window.get());
   d.run();
