@@ -69,6 +69,14 @@ History* History::handle_load(XML_Helper* helper)
       return (new History_Score(helper));
     case PLAYER_VANQUISHED:
       return (new History_PlayerVanquished(helper));
+    case DIPLOMATIC_PEACE:
+      return (new History_DiplomacyPeace(helper));
+    case DIPLOMATIC_WAR:
+      return (new History_DiplomacyWar(helper));
+    case DIPLOMATIC_TREACHERY:
+      return (new History_DiplomacyTreachery(helper));
+    case HERO_FINDS_ALLIES:
+      return (new History_HeroFindsAllies(helper));
     }
 
   return 0;
@@ -126,6 +134,22 @@ History* History::copy(const History* a)
       return 
 	(new History_PlayerVanquished
 	 (*dynamic_cast<const History_PlayerVanquished*>(a)));
+    case DIPLOMATIC_PEACE:
+      return 
+	(new History_DiplomacyPeace
+	 (*dynamic_cast<const History_DiplomacyPeace*>(a)));
+    case DIPLOMATIC_WAR:
+      return 
+	(new History_DiplomacyWar
+	 (*dynamic_cast<const History_DiplomacyWar*>(a)));
+    case DIPLOMATIC_TREACHERY:
+      return 
+	(new History_DiplomacyTreachery
+	 (*dynamic_cast<const History_DiplomacyTreachery*>(a)));
+    case HERO_FINDS_ALLIES:
+      return 
+	(new History_HeroFindsAllies
+          (*dynamic_cast<const History_HeroFindsAllies*>(a)));
     }
 
   return 0;
@@ -761,3 +785,187 @@ bool History_PlayerVanquished::save(XML_Helper* helper) const
 
   return retval;
 }
+
+//-----------------------------------------------------------------------------
+//History_DiplomacyPeace
+
+History_DiplomacyPeace::History_DiplomacyPeace()
+:History(History::DIPLOMATIC_PEACE), d_opponent_id(0)
+{
+}
+
+History_DiplomacyPeace::History_DiplomacyPeace(XML_Helper* helper)
+:History(History::DIPLOMATIC_PEACE)
+{
+  helper->getData(d_opponent_id, "opponent_id");
+}
+
+History_DiplomacyPeace::~History_DiplomacyPeace()
+{
+}
+
+std::string History_DiplomacyPeace::dump() const
+{
+  std::stringstream s;
+
+  s <<"peace has been won with player " << d_opponent_id;
+  s <<"\n";
+
+  return s.str();
+}
+
+bool History_DiplomacyPeace::save(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  retval &= helper->openTag("history");
+  retval &= helper->saveData("type", d_type);
+  retval &= helper->saveData("opponent_id", d_opponent_id);
+  retval &= helper->closeTag();
+
+  return retval;
+}
+
+bool History_DiplomacyPeace::fillData(Player *opponent)
+{
+  d_opponent_id = opponent->getId();
+  return true;
+}
+
+//-----------------------------------------------------------------------------
+//History_DiplomacyWar
+
+History_DiplomacyWar::History_DiplomacyWar()
+:History(History::DIPLOMATIC_WAR), d_opponent_id(0)
+{
+}
+
+History_DiplomacyWar::History_DiplomacyWar(XML_Helper* helper)
+:History(History::DIPLOMATIC_WAR)
+{
+  helper->getData(d_opponent_id, "opponent_id");
+}
+
+History_DiplomacyWar::~History_DiplomacyWar()
+{
+}
+
+std::string History_DiplomacyWar::dump() const
+{
+  std::stringstream s;
+
+  s <<"war has been declared with player " << d_opponent_id;
+  s <<"\n";
+
+  return s.str();
+}
+
+bool History_DiplomacyWar::save(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  retval &= helper->openTag("history");
+  retval &= helper->saveData("type", d_type);
+  retval &= helper->saveData("opponent_id", d_opponent_id);
+  retval &= helper->closeTag();
+
+  return retval;
+}
+
+bool History_DiplomacyWar::fillData(Player *opponent)
+{
+  d_opponent_id = opponent->getId();
+  return true;
+}
+
+//-----------------------------------------------------------------------------
+//History_DiplomacyTreachery
+
+History_DiplomacyTreachery::History_DiplomacyTreachery()
+:History(History::DIPLOMATIC_TREACHERY), d_opponent_id(0)
+{
+}
+
+History_DiplomacyTreachery::History_DiplomacyTreachery(XML_Helper* helper)
+:History(History::DIPLOMATIC_TREACHERY)
+{
+  helper->getData(d_opponent_id, "opponent_id");
+}
+
+History_DiplomacyTreachery::~History_DiplomacyTreachery()
+{
+}
+
+std::string History_DiplomacyTreachery::dump() const
+{
+  std::stringstream s;
+
+  s <<"treachery on player " << d_opponent_id;
+  s <<"\n";
+
+  return s.str();
+}
+
+bool History_DiplomacyTreachery::save(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  retval &= helper->openTag("history");
+  retval &= helper->saveData("type", d_type);
+  retval &= helper->saveData("opponent_id", d_opponent_id);
+  retval &= helper->closeTag();
+
+  return retval;
+}
+
+bool History_DiplomacyTreachery::fillData(Player *opponent)
+{
+  d_opponent_id = opponent->getId();
+  return true;
+}
+
+//-----------------------------------------------------------------------------
+//History_HeroFindsAllies
+
+History_HeroFindsAllies::History_HeroFindsAllies()
+:History(History::HERO_FINDS_ALLIES), d_hero("")
+{
+}
+
+History_HeroFindsAllies::History_HeroFindsAllies(XML_Helper* helper)
+:History(History::HERO_FINDS_ALLIES)
+{
+  helper->getData(d_hero, "hero");
+}
+
+History_HeroFindsAllies::~History_HeroFindsAllies()
+{
+}
+
+std::string History_HeroFindsAllies::dump() const
+{
+  std::stringstream s;
+
+  s <<"hero " << d_hero<< " finds some allies\n";
+
+  return s.str();
+}
+
+bool History_HeroFindsAllies::save(XML_Helper* helper) const
+{
+  bool retval = true;
+
+  retval &= helper->openTag("history");
+  retval &= helper->saveData("type", d_type);
+  retval &= helper->saveData("hero", d_hero);
+  retval &= helper->closeTag();
+
+  return retval;
+}
+
+bool History_HeroFindsAllies::fillData(Hero *hero)
+{
+  d_hero = hero->getName();
+  return true;
+}
+
