@@ -108,14 +108,19 @@ class Player: public sigc::trackable
         //! The available player types. Needed when loading a player.
         enum Type {HUMAN = 0, AI_FAST = 1, AI_DUMMY = 2, AI_SMART = 4 };
 
+	//! Every player has a diplomatic state with every other player.
+	enum DiplomaticState {AT_PEACE = 1, AT_WAR_IN_FIELD = 2, AT_WAR = 3};
+	enum DiplomaticProposal {NO_PROPOSAL = 0, PROPOSE_PEACE = 1, 
+	  PROPOSE_WAR_IN_FIELD = 2, PROPOSE_WAR = 3 };
+
         /** Default constructor
           *
           * @param name         the name of the player
           * @param armyset      the player's armyset
           * @param color        the player's main color
           */
-        Player (std::string name, Uint32 armyset, SDL_Color color, Type type,
-		int player_no = -1);
+        Player (std::string name, Uint32 armyset, SDL_Color color, int width,
+		int height, Type type, int player_no = -1);
 
         //! Copy constructor
         Player(const Player&);
@@ -133,8 +138,10 @@ class Player: public sigc::trackable
           * @param armyset  the armyset of the player
           * @param color    the color of the player
           * @param type     the player's type
+          * @param width    the width of the player's fogmap
+          * @param height   the height of the player's fogmap
           */
-        static Player* create(std::string name, Uint32, SDL_Color color, Type type);
+        static Player* create(std::string name, Uint32, SDL_Color color, int width, int height, Type type);
         
         /** copies a player to a different type
           * 
@@ -217,6 +224,55 @@ class Player: public sigc::trackable
 	//! Calculates the upkeep
 	void calculateUpkeep();
 
+	//! Declare a new diplomatic state wrt another player
+	void declareDiplomacy(DiplomaticState state, Player *player);
+
+	//! Query the diplomatic state wrt another player
+	DiplomaticState getDiplomaticState (Player *player);
+
+	//! Query the diplomatic proposal we're making wrt another player
+	DiplomaticProposal getDiplomaticProposal (Player *player);
+
+	//! Propose a new diplomatic state wrt another player
+	void proposeDiplomacy (DiplomaticProposal proposal, Player *player);
+
+	//! Set this player's rank in diplomatic matters.  Starts at 1.
+	void setDiplomaticRank (Uint32 rank) {d_diplomatic_rank = rank;};
+
+	//! What diplomatic rank does this player have? As a number.
+	//! Starts at 1.
+	Uint32 getDiplomaticRank () {return d_diplomatic_rank;};
+
+	//! What rank do we have? As a name.
+	std::string getDiplomaticTitle() {return d_diplomatic_title;};
+
+	//! Set the rank as a name.
+	void setDiplomaticTitle (std::string title) {d_diplomatic_title = title;};
+	//! Negotiate diplomatic talks.
+	DiplomaticState negotiateDiplomacy (Player *player);
+
+	//! Get the diplomatic score wrt another player
+	Uint32 getDiplomaticScore (Player *p);
+
+	//! Make your diplomatic view of another player increase
+	void improveDiplomaticRelationship (Player *p, Uint32 amount);
+
+	//! Make all other players diplomatic view of you increase,
+	//! excepting one player (if not null).
+	void improveDiplomaticRelationship (Uint32 amount, Player *except);
+
+	//! Make your diplomatic view of another player decrease
+	void deteriorateDiplomaticRelationship (Player *, Uint32 amount);
+
+	//! Make all other players diplomatic view of you worsen
+	void deteriorateDiplomaticRelationship (Uint32 amount);
+
+	//! Make players you are at STATE with you think less of PLAYER.
+	void deteriorateAlliesRelationship(Player *player, Uint32 amount, Player::DiplomaticState state);
+
+	//! Make players who are at STATE with PLAYER think better of you.
+	void improveAlliesRelationship(Player *player, Uint32 amount, Player::DiplomaticState state);
+
         //! Returns the main color of the player
         SDL_Color getColor() const {return d_color;}
 
@@ -372,6 +428,17 @@ class Player: public sigc::trackable
                                          bool ruin) =0;
         virtual Fight::Result stackRuinFight(Stack** attacker, Stack** defender) =0;
 
+	/** Decide if we perform treachery on a friendly player.
+	 *
+	 * @param stack             my stack considering the treachery
+	 * @param player            the friendly player.
+	 * @param pos               the place on the map being targetted.
+	 * @param state             the state we end up in if we decide yes.
+	 *
+	 * @return true if we decided to be treacherous.  false otherwise.
+	 */
+	virtual bool treachery (Stack *stack, Player *player, Vector <int> pos) =0;
+
         /** A stack searches a ruin. The stack should contain a hero.
           *
           * @param s                the stack which searches the ruin
@@ -396,9 +463,10 @@ class Player: public sigc::trackable
          *
          * @param s                 the visiting stack
          * @param t                 the visited temple
+         * @param except_raze       don't give out a raze quest
          * @return the quest we got or 0 on error
          */
-        virtual Quest* stackGetQuest(Stack* s, Temple* t) = 0;
+        virtual Quest* stackGetQuest(Stack* s, Temple* t, bool except_raze) = 0;
         
         /** Called to ask the military advisor about what would happen 
 	  * if the stack attacked the tile.
@@ -407,7 +475,8 @@ class Player: public sigc::trackable
 	  * @param tile     the tile to attack (could be a city, or a stack)
           * @return percent chance to win
           */
-        virtual float stackFightAdvise(Stack* s, Vector<int> tile) =0;
+        virtual float stackFightAdvise(Stack* s, Vector<int> tile,
+				       bool intense_combat) =0;
 
         /** Occupy a city (i.e. change the owner to yourself)
           *
@@ -510,12 +579,20 @@ class Player: public sigc::trackable
           */
         sigc::signal<void, City*> sinvadingCity;
 
+	/** Signal raised when a stack is considering an act of treachery.
+	 */
+        sigc::signal<bool, Stack *, Player *, Vector<int> > streacheryStack;
+
+	/** Signal raised when a human player is deciding.
+	 */
+        sigc::signal<bool, Stack *, Player *, Vector<int> > streachery;
+
         //! Signal raised whenever a player has conquered a city. This is the
         //! signal you should use for further actions.
         sigc::signal<void, City*, Stack*> soccupyingCity;
 
         
-        //! Signal raised when a hero is recruited
+        //! Signal raised when a hero is recruited.  Human players only.
         sigc::signal<bool, Hero*, City *, int>         srecruitingHero;
         //! Signal raised when an army advances a level; may return stat to raise
         sigc::signal<Army::Stat, Army*>        snewLevelArmy;
@@ -549,6 +626,9 @@ class Player: public sigc::trackable
 	// emitted when a fight is started, parameters are in the fight object,
 	// so should the results be
         sigc::signal<void, Fight &> fight_started;
+
+	//emitted after we attack a city
+        sigc::signal<void, City *, Fight::Result> cityfight_finished;
 	
 	// emitted when a fight in a ruin is started
         sigc::signal<void, Stack *, Stack *> ruinfight_started;
@@ -580,11 +660,19 @@ class Player: public sigc::trackable
 	std::list<Uint32> d_fight_order; //for each army in armyset, a number
 	Uint32 d_triumph[MAX_PLAYERS][5]; // 5 is max TriumphType + 1
 	Uint32 d_upkeep; //how much we paid out last turn
+	DiplomaticState d_diplomatic_state[MAX_PLAYERS];
+	Uint32 d_diplomatic_rank;
+	std::string d_diplomatic_title;
+	DiplomaticProposal d_diplomatic_proposal[MAX_PLAYERS];
+	Uint32 d_diplomatic_score[MAX_PLAYERS];
 
 
     private:
         //! Loads the subdata of a player (actions and stacklist)
         bool load(std::string tag, XML_Helper* helper);
+
+	//! assists in scorekeeping for diplomacy
+	void alterDiplomaticRelationshipScore (Player *player, int amount);
 };
 
 extern sigc::signal<void, Player::Type>  sendingTurn;
