@@ -38,7 +38,6 @@
 #include "FogMap.h"
 #include "xmlhelper.h"
 #include "ruinlist.h"
-#include "GameScenario.h"
 #include "game-parameters.h"
 #include "signpost.h"
 #include "history.h"
@@ -48,9 +47,9 @@ using namespace std;
 //#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
 #define debug(x)
 
-RealPlayer::RealPlayer(string name, Uint32 armyset, SDL_Color color,
-		       Player::Type type, int player_no)
-    :Player(name, armyset, color, type, player_no)
+RealPlayer::RealPlayer(string name, Uint32 armyset, SDL_Color color, int width,
+		       int height, Player::Type type, int player_no)
+    :Player(name, armyset, color, width, height, type, player_no)
 {
 }
 
@@ -108,10 +107,6 @@ bool RealPlayer::recruitHero(Hero* hero, City *city, int cost)
     // for the realplayer, this function also just raises a signal and looks
     // what to do next.
 
-    History_HeroEmerges *item = new History_HeroEmerges();
-    item->fillData(hero, city);
-    d_history.push_back(item);
-
     return srecruitingHero.emit(hero, city, cost);
 }
 
@@ -305,6 +300,16 @@ MoveResult *RealPlayer::stackMove(Stack* s, Vector<int> dest, bool follow)
         //first fight_city to avoid ambiguity with fight_army
         if (city && (city->getPlayer() != this) && (!city->isBurnt()))
         {
+	    if (this->getDiplomaticState (city->getPlayer()) != AT_WAR)
+	      {
+		if (streacheryStack.emit (s, city->getPlayer(), 
+					  city->getPos()) == false)
+		  {
+		    s->getPath()->flClear();
+		    MoveResult *moveResult = new MoveResult(false);
+		    return moveResult;
+		  }
+	      }
             Fight::Result result;
             MoveResult *moveResult = new MoveResult(true);
 	    if (stackMoveOneStep(s))
@@ -317,6 +322,7 @@ MoveResult *RealPlayer::stackMove(Stack* s, Vector<int> dest, bool follow)
                 // and start a fight with this dummy stack.
                 if (!target)
                     target = new Stack(city->getPlayer(), pos);
+ 
                 result = stackFight(&s, &target, false);
                 if (target && target->empty())
                     delete target;
@@ -344,34 +350,8 @@ MoveResult *RealPlayer::stackMove(Stack* s, Vector<int> dest, bool follow)
             }
             else
             {
-                // we didn't suceed in defeating the defenders
-                //if this is a neutral city, and we're playing with 
-                //active neutral cities, AND it hasn't already been attacked
-                //then it's production gets turned on
-                Player *neu = city->getPlayer(); //neutral player
-                if (GameScenario::s_neutral_cities == GameParameters::ACTIVE &&
-                    neu == Playerlist::getInstance()->getNeutral() &&
-                    city->getProductionIndex() == -1)
-                {
-                  //great, then let's turn on the production.
-                  //well, we already made a unit, and we want to produce more
-                  //of it.
-                  Stack *o = neu->getStacklist()->getObjectAt(city->getPos());
-                  if (o)
-                    {
-                      int army_type = o->getStrongestArmy()->getType();
-                      for (int i = 0; i < 4; i++)
-                        {
-                          if (city->getArmytype(i) == army_type)
-                            {
-                              // hey, we found the droid we were looking for
-                              city->setProduction(i);
-                              break;
-                            }
-                        }
-                    }
-                }
             }
+	    cityfight_finished(city, result);
             moveResult->setStepCount(stepCount);
             supdatingStack.emit(0);
             
@@ -397,6 +377,16 @@ MoveResult *RealPlayer::stackMove(Stack* s, Vector<int> dest, bool follow)
         //enemy stack => fight
         else if (target)
         {
+	  if (this->getDiplomaticState (target->getPlayer()) == AT_PEACE)
+	    {
+	      if (streacheryStack.emit (s, target->getPlayer(), 
+					target->getPos()) == false)
+		{
+		  s->getPath()->flClear();
+		  MoveResult *moveResult = new MoveResult(false);
+		  return moveResult;
+		}
+	    }
             MoveResult *moveResult = new MoveResult(true);
         
             Fight::Result result = stackFight(&s, &target, false);
@@ -425,6 +415,14 @@ MoveResult *RealPlayer::stackMove(Stack* s, Vector<int> dest, bool follow)
         moveResult->setStepCount(stepCount);
         return moveResult;
     }
+    else if (s->getPath()->size() == 1 && s->enoughMoves() == false)
+    {
+      /* if we can't attack a city, don't remember it in the stack's path. */
+        Vector<int> pos = **(s->getPath()->begin());
+        City* city = Citylist::getInstance()->getObjectAt(pos);
+	if (city && city->getPlayer() != this)
+	  s->getPath()->flClear();
+    }
 
     //If there is another stack where we landed, join it. We can't have two
     //stacks share the same maptile
@@ -569,6 +567,11 @@ Fight::Result RealPlayer::stackRuinFight (Stack **attacker, Stack **defender)
     return result;
 }
 
+bool RealPlayer::treachery (Stack *stack, Player *player, Vector <int> pos)
+{
+  return streachery.emit(stack, player, pos);
+}
+
 Fight::Result RealPlayer::stackFight(Stack** attacker, Stack** defender, bool ruin) 
 {
     debug("stackFight: player = " << getName()<<" at position "
@@ -581,10 +584,11 @@ Fight::Result RealPlayer::stackFight(Stack** attacker, Stack** defender, bool ru
 
     // I suppose, this should be always true, but one can never be sure
     bool attacker_active = *attacker == d_stacklist->getActivestack();
+    Player* pa = (*attacker)->getPlayer();
 
     Fight fight(*attacker, *defender);
-    fight_started.emit(fight);
 
+    fight_started.emit(fight);
     // cleanup
     
     // add a fight item about the combat
@@ -968,6 +972,7 @@ Reward* RealPlayer::stackSearchRuin(Stack* s, Ruin* r)
 	   r->populateWithRandomReward();
        }
 
+     retReward = r->getReward();
      ssearchingRuin.emit(r, s, retReward);
 
      r->setSearched(true);
@@ -977,7 +982,7 @@ Reward* RealPlayer::stackSearchRuin(Stack* s, Ruin* r)
      d_actions.push_back(item);
 
      supdatingStack.emit(0);
-     return r->getReward();
+     return retReward;
 }
 
 int RealPlayer::stackVisitTemple(Stack* s, Temple* t)
@@ -1007,7 +1012,7 @@ int RealPlayer::stackVisitTemple(Stack* s, Temple* t)
   return count;
 }
 
-Quest* RealPlayer::stackGetQuest(Stack* s, Temple* t)
+Quest* RealPlayer::stackGetQuest(Stack* s, Temple* t, bool except_raze)
 {
   QuestsManager *qm = QuestsManager::getInstance();
   debug("Realplayer::stackGetQuest")
@@ -1027,7 +1032,8 @@ Quest* RealPlayer::stackGetQuest(Stack* s, Temple* t)
   Quest* q=0;
   if (s->getFirstHero())
     {
-      q = qm->createNewQuest(s->getFirstHero()->getId());
+      q = qm->createNewQuest
+	(s->getFirstHero()->getId(), except_raze);
     }
 
   // couldn't assign a quest for various reasons
@@ -1195,6 +1201,7 @@ bool RealPlayer::cityRaze(City* c)
   history->fillData(c);
   d_history.push_back(history);
 
+  c->conquer(this);
   c->setBurnt(true);
 
   supdatingCity.emit(c);
@@ -1263,6 +1270,7 @@ bool RealPlayer::giveReward(Stack *s, Reward *reward)
 
 	    Reward_Allies::addAllies(s->getPlayer(), s->getPos(), a,
 				     dynamic_cast<Reward_Allies*>(reward)->getNoOfAllies());
+    
 	  }
 	break;
       case Reward::ITEM:
@@ -1518,9 +1526,10 @@ void RealPlayer::tallyTriumph(Player *p, TriumphType type)
   d_triumph[id][type]++;
 }
 
-float RealPlayer::stackFightAdvise(Stack* s, Vector<int> tile)
+float RealPlayer::stackFightAdvise(Stack* s, Vector<int> tile, 
+				   bool intense_combat)
 {
-  float percent = 0.0;//(rand() % 100) * 1.0;
+  float percent = 0.0;
         
   City* city = Citylist::getInstance()->getObjectAt(tile);
   Stack* target = Stacklist::getObjectAt(tile);
@@ -1533,7 +1542,7 @@ float RealPlayer::stackFightAdvise(Stack* s, Vector<int> tile)
   for (unsigned int i = 0; i < 100; i++)
     {
       Fight fight(s, target, Fight::FOR_KICKS);
-      fight.battle();
+      fight.battle(intense_combat);
       if (fight.getResult() == Fight::ATTACKER_WON)
 	percent += 1.0;
     }
