@@ -370,16 +370,27 @@ Uint32 Stack::getMaxSight() const
 
 void Stack::nextTurn()
 {
+  Uint32 movement_multiplier = 1;
   setParked(false);
-  for (iterator it = begin(); it != end(); ++it)
-    {
-      (*it)->resetMoves();
-      // TODO: should be moved in a more appropriate place => class Player
-      if (d_player)
-	d_player->withdrawGold((*it)->getUpkeep());
-      (*it)->heal();
-    }
 
+  //count the number of items that double the movement in the stack.
+  for (const_iterator it = begin(); it != end(); it++)
+    if ((*it)->isHero())
+      {
+	std::list<Item*> backpack = dynamic_cast<Hero*>((*it))->getBackpack();
+	std::list<Item*>::const_iterator item;
+	for (item = backpack.begin(); item != backpack.end(); item++)
+	  {
+	    if ((*item)->getBonus(Item::DOUBLEMOVESTACK))
+	      movement_multiplier++;
+	  }
+      }
+
+  //set the multipler on all armies in the stack
+  for (const_iterator it = begin(); it != end(); it++)
+    (*it)->setStat(Army::MOVES_MULTIPLIER, movement_multiplier);
+
+  //now let's see if we have any items that give us gold per city
   for (const_iterator it = begin(); it != end(); it++)
     if ((*it)->isHero())
       {
@@ -388,8 +399,6 @@ void Stack::nextTurn()
 	for (item = backpack.begin(); item != backpack.end(); item++)
 	  {
 	    Player *p = d_player;
-	    if ((*item)->getBonus(Item::DOUBLEMOVESTACK))
-	      (*it)->setStat(Army::MOVES, (*it)->getMoves() * 2);
 	    if ((*item)->getBonus(Item::ADD2GOLDPERCITY))
 	      p->addGold(2 * Citylist::getInstance()->countCities(p));
 	    if ((*item)->getBonus(Item::ADD3GOLDPERCITY))
@@ -402,8 +411,19 @@ void Stack::nextTurn()
       }
   if (d_defending == true)
     setFortified(true);
+
+  for (iterator it = begin(); it != end(); ++it)
+    {
+      (*it)->resetMoves();
+      // TODO: should be moved in a more appropriate place => class Player
+      if (d_player)
+	d_player->withdrawGold((*it)->getUpkeep());
+      (*it)->heal();
+    }
+
   //recalculate paths
   getPath()->recalculate(this);
+
 }
 
 bool Stack::save(XML_Helper* helper) const
